/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// LiveDemo.h: interface for the CLiveObject class.
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_LIVEDEMO_H__3FB62209_C384_4185_9A1C_B021214C85F1__INCLUDED_)
#define AFX_LIVEDEMO_H__3FB62209_C384_4185_9A1C_B021214C85F1__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Definition
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#define		MAX_MTD_ROW_COUNT			(18)
#define		MAX_MTD_MAP_INFO_INDEX		(MAX_MTD_ROW_COUNT*4)

#define		MAX_ALARMIN_COUNT			8
#define		MAX_ALARMOUT_COUNT			8

#define		LEN_AUDIOOUT_WAVEFORMBUF	5
#define		LEN_AUDIOIN_WAVEFORMBUF		2

#define		AUDIO_16KSAMPLE_PER_SEC		16000
#define		AUDIO_8KSAMPLE_PER_SEC		8000
#define		AUDIO_BITS_FOR_SAMPLE		16
#define		AUDIO_DATABLOCK_SIZE		4000

#define		MAX_VIDEO_BUFFER_SIZE		1400

#define		LEN_STARTCODE				5
#define		LEN_PACKETSIZE				4
#define		LEN_ENDCODE					5
#define		LEN_SLICESIZE				1400

#define		MAX_CHANNELS				5

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Resolution
#define	WIDTH_1080i				(1920)
#define HEIGHT_1080i			(1080)

#define	WIDTH_720p				(1280)
#define HEIGHT_720p				(720)

#define	WIDTH_D1_HD				(720)
#define HEIGHT_D1_NTSC			(480)
#define HEIGHT_D1_PAL			(576)
#define HEIGHT_HALF_D1_PAL		(288)
#define HEIGHT_HALF_D1_NTSC		(240)

#define	WIDTH_4CIF_SD			(704)
#define HEIGHT_4CIF_PAL			(576)
#define HEIGHT_4CIF_NTSC		(480)
#define HEIGHT_HALF_4CIF_PAL	(288)
#define HEIGHT_HALF_4CIF_NTSC	(240)

#define WIDTH_CIF				(352)
#define HEIGHT_CIF_NTSC			(240)
#define HEIGHT_CIF_PAL			(288)

#define WIDTH_QCIF				(176)
#define HEIGHT_QCIF_NTSC		(128)
#define HEIGHT_QCIF_PAL			(144)

#define WIDTH_VGA				(640)
#define HEIGHT_VGA				(480)

#define WIDTH_QVGA				(320)
#define HEIGHT_QVGA				(240)

#define WIDTH_QQVGA				(160)
#define HEIGHT_QQVGA			(120)

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// enum
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

enum eDM365CameraResolution { vr1080, vr720p, vr4CIF, vrVGA, vrCIF, vrQVGA, vrQQVGA, vrHALF_D1 };
enum eCodecType				{ ctMJPEG_MD_PENTA, ctMPEG4_MD_PENTA, ctMPEG4_MD_DM365, ctH264_MD_DM365, ctMJPEG_DM365 };
enum eCameraMountType		{ cmNormal, cmFlip };
enum eCamOutputType			{ coNone = -1,	coNTSC, coPAL };
enum eChannel				{ CH_VIDEO1, CH_VIDEO2 };
enum eInput					{ INPUT1, INPUT2, INPUT3, INPUT4, INPUT5, INPUT6, INPUT7, INPUT8 };
enum eOutput				{ OUTPUT1, OUPUT2, OUTPUT3, OUTPUT4, OUTPUT5, OUTPUT6 };
enum eMotionMode			{ ALL_OFF, MD_ON, MT_ON };
enum eAuthentication		{ NONE, ADMINISTRATOR, GUEST };
enum eAudioChannelType		{ AT_MONO = 1, AT_STEREO };
enum eConnectionStatus		{ STATUS_NONE = -1, STATUS_CONNECTED = 0, STATUS_LOGON_FAIL, STATUS_LOGON, STATUS_LOGOFF };

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Include File
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "vfw.h"
#pragma comment(lib,"vfw32.lib")

#include "../include/AudCoderDLL.h"
#include "../include/ProbeNetCtrlrDll.h"
#pragma comment(lib, "..\\lib/ProbeNetCtrlr.lib")

#include "../include/ProbeNetCtrlrDef.h"
#pragma comment(lib, "..\\lib/GLOBALUTILS.lib")

#include "../include/GLOBALUTILS_DLL.h"

#include "VideoCodec.h"
#include "VideoView.h"

#pragma comment(lib,"winmm.lib")

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Structure
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma pack(1)
struct stStreamSlice
{
	BYTE	_szStartCode [LEN_STARTCODE];
	BYTE	_szPacketSize[LEN_PACKETSIZE];
	BYTE	_btIsStream;
	BYTE	_btCodecType;
	DWORD	_dwFrameNum;
	DWORD	_dwFrameSize;
	WORD	_wSliceNum;
	WORD	_wSliceSize;
	//  Channel ϱ  Ⱦ  ü.
	//WORD	_wEncodeOption;				// I-Reference, Standard
	BYTE	_btChannelIndex;			// Channel Index
	BYTE	_btreserved;				
	// End.
	DWORD	_dwGopIndex;				// Group Of Pictures Index : IFrame Count
	WORD	_wGopSubIndex;				// 0~15 
	WORD	_wIsNotKeyFrame;
	BYTE	_szData   [LEN_SLICESIZE];
	BYTE	_szEndCode[LEN_ENDCODE];
};
#pragma pack()

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class CLiveChannel
{
private:
	HANDLE			m_hConnection;
	HANDLE			m_hAudioMicconnection;

	DWORD			m_dwCtrlCode;
	BYTE			*m_lpVideoData;
	BYTE            *m_lpMotionInfo;
	DWORD           m_dwMotionSize;

	DWORD           m_dwSensitivity;
	
	DWORD			m_dwVideoOffset;
	DWORD			m_dwVideoFrameSize;
	
	BYTE			*m_pOutImage;
	BYTE			*m_pOutImageTemp;
	
	int				m_nWidth;
	int				m_nHeight;
	
	int				m_nCodecType;
	
	BOOL			m_IsControlConnected;
	BOOL			m_IsVideoConnected;
	BOOL			m_IsAudioSpeakerConnected;
	BOOL			m_IsAudioMicConnected;
	
	int				m_nControlStatus;
	int				m_nVideoStatus;
	int				m_nAudioSpeakerStatus;
	int				m_nAudioMicStatus;

	DWORD			m_dwVideoSliceNum;
	DWORD			m_dwVideoFrameNum;
	
	int				m_IsIFrame;
	DWORD			m_dwIFrameNumber;

	//..................................................

	BOOL			m_bEventAlarmInput1;
	BOOL			m_bEventAlarmInput2;
	BOOL			m_bEventMotion;
	int				m_nMotionMode;
	int				m_nAuthentication;

	BOOL			m_ISMPEG4DecodeInit;
	
	//..................................................
	// Audio
	DWORD			m_dwAudioOffset;
	DWORD			m_dwAudioFrameSize;
	
	AUDIO_DATA_PACK	*m_AudiodataPack;
	
	BYTE			*m_lpAudioSpeakerData;
	WAVEHDR			*m_arpWaveOutHdr[LEN_AUDIOOUT_WAVEFORMBUF];
	HANDLE			m_hAudioDecoder;
	int             m_nAudioCodecType;
	HWAVEOUT		m_hWaveOut;
	int				m_nWaveOutBufIndex;
	BOOL			m_bOnSpeaker;
	WAVEFORMATEX	m_WaveOutFormatData;

	BYTE			*m_lpAudioMicData;
	WAVEHDR			*m_arpWaveInHdr[LEN_AUDIOIN_WAVEFORMBUF];
	HANDLE			m_hAudioEncoder;
	HWAVEIN			m_hWaveIn;
	int				m_nWaveInBufIndex;
	WAVEFORMATEX	m_WaveInFormatData;
	DWORD			m_dwFrameNum;
	BOOL			m_bRecording;

	//..................................................

	WORD			m_wPTZSpeed;
	
	//..................................................

public:
	CVideoView	    *m_Render;
	CVideoCodec		*m_Decoder;
	BOOL			 m_isFirstFrame;
	
	//..................................................
	
	CAMERA_CONFIG	m_CameraConfig;
	VIDEO_DATA_PACK *m_pRcvVideoData;

	BOOL			*m_IsDecodingSuccess;
	WORD			*m_pwPan;
	WORD			*m_pwTilt;
	WORD			*m_pwZoom;
	char			*m_cVOLHeader;
	
	BYTE			*m_byAlarmIn;
	BYTE			*m_byAlarmOut;

	//..................................................

	int				m_nResolutionID;

	//..................................................

public:
	CLiveChannel();
	virtual ~CLiveChannel();
	
	//..................................................

public:
	void End();

	void InitAudioSpeaker();
	void PlayAudioSpeaker(int nCodecType = AUDIO_CODEC_G711_16K);
	void StopAudioSpeaker();
	void TerminateAudioSpeaker();
	void AudioSpeakerOn( BOOL bFlag );

	void InitAudioMic();
	void PlayAudioMic();
	void StopAudioMic();
	void TerminateAudioMic();
	void EncodeAudio( WAVEHDR *pWaveInHdr );

	void OnConnect( char *szAddr, char *szControlPort, char *szVideoPort, char *szAudioTPort, char *szAudioRPort, char *szUserId, char *szPasswd );
	void OnDisConnect();

	void RefreshDecoder( int nCodecType );

	void SetMotionDetectionMode( int nMode );
	void SetPTZSpeed( WORD wSpeed );
	void SetPTZMove( int nPan, int nTilt, double nZoom );
	void GetPTZCoordinates();
	int SetEncoderSetupInfo( ENCODE_SETUP_INFO *lpData );
    void SetOutput( BOOL bOn, int nOutput ) ;
	void SetVOLHeader( int nResolutionIndex );
	int Make_header( char *pVOLHeader, int width, int height );
	
	inline BOOL GetRecordingStatus()		{ return m_bRecording; };
	inline BOOL GetControlConnection()		{ return m_IsControlConnected; };
	inline BOOL GetVideoConnection()		{ return m_IsVideoConnected; };
	inline BOOL GetAudioSpeakerConnection()	{ return m_IsAudioSpeakerConnected; };
	inline BOOL GetAudioMicConnection()		{ return m_IsAudioMicConnected; };
	inline int  GetControlStatus()			{ return m_nControlStatus; };
	inline int  GetVideoStatus()			{ return m_nVideoStatus; };
	inline int  GetAudioSpeakerStatus()		{ return m_nAudioSpeakerStatus; };
	inline int  GetAudioMicStatus()			{ return m_nAudioMicStatus; };

	inline HANDLE GetHandle()				{ return m_hConnection; };
	inline BYTE *GetOutImage()				{ return m_pOutImage; };
	inline int	GetOutImageWidth()			{ return m_nWidth; };
	inline int  GetOutImageHeight()			{ return m_nHeight; };
	inline CAMERA_CONFIG GetCameraConfig()	{ return m_CameraConfig; };
	inline BOOL GetAlarmInput1Status()		{ return m_bEventAlarmInput1; };
	inline BOOL GetAlarmINput2Status()		{ return m_bEventAlarmInput2; };
	inline BOOL GetMotionEvent()			{ return m_bEventMotion; };
	inline int	GetAuthentication()			{ return m_nAuthentication; };

	WAVEFORMATEX	GetAudioInFormat()		{ return m_WaveInFormatData; };
	WAVEFORMATEX	GetAudioOutFormat()		{ return m_WaveOutFormatData; };

	BOOL GetEncoderSetupInfo( int nChannel, ENCODE_SETUP_INFO *lpData, BOOL bRefreshData );

	void GetAlarmInOutStatus();


	void SaveMotionArea();
	void SaveMotionSensitivity(int nValue);
	int  GetMotionSensitivity();

	//..................................................

	void ButtonLeftUp();
	void ButtonUp();
	void ButtonRightup();
	void ButtonLeft();
	void ButtonRight();
	void ButtonLeftdown();
	void ButtonDown();
	void ButtonRightdown();
	void ButtonStop();
	void ButtonZoomIn();
	void ButtonZoomOut();
	
	//..................................................

	void ConnectVideo( BOOL bConnect );
	void ConnectSpeaker( BOOL bConnect );
	void ConnectMic( BOOL bConnect );

	//..................................................

	static void _ON_RECV_CTRL_DATA(HANDLE hCallback, char *szCommandCode, DWORD_PTR param1, DWORD_PTR param2);
	static void _ON_RECV_VIDEO_CTRL_DATA(HANDLE hCallback, char *szCommandCode, DWORD_PTR param1, DWORD_PTR param2);
	static void _ON_RECV_VIDEO_DATA(HANDLE hCallback, VIDEO_DATA_PACK *lpData, DWORD_PTR param1, DWORD_PTR param2);
	static void _ON_RECV_AUDIO_SPEAKER_CTRL_DATA(HANDLE hCallback, char *szCommandCode, DWORD_PTR param1, DWORD_PTR param2);
	static void _ON_RECV_AUDIO_SPEAKER_DATA(HANDLE hCallback, AUDIO_DATA_PACK *lpData);
	static void _ON_RECV_AUDIO_MIC_CTRL_DATA(HANDLE hCallback, char *szCommandCode, DWORD_PTR param1, DWORD_PTR param2);
	static void _ON_ERROR_CTRL(HANDLE hCallback, int nErrorCode);
	static void _ON_ERROR_VIDEO(HANDLE hCallback, int nErrorCode);
	static void _ON_ERROR_AUDIO_SPEAKER(HANDLE hCallback, int nErrorCode);
	static void _ON_ERROR_AUDIO_MIC(HANDLE hCallback, int nErrorCode);
	
	void ON_RECV_CTRL_DATA(char *szCommandCode, DWORD_PTR param1, DWORD_PTR param2);
	void ON_RECV_VIDEO_CTRL_DATA(char *szCommandCode, DWORD_PTR param1, DWORD_PTR param2);
	void ON_RECV_VIDEO_DATA(VIDEO_DATA_PACK *lpData, DWORD_PTR param1, DWORD_PTR param2);
	void ON_RECV_AUDIO_SPEAKER_CTRL_DATA(char *szCommandCode, DWORD_PTR param1, DWORD_PTR param2);
	void ON_RECV_AUDIO_SPEAKER_DATA(AUDIO_DATA_PACK *lpData);
	void ON_RECV_AUDIO_MIC_CTRL_DATA(char *szCommandCode, DWORD_PTR param1, DWORD_PTR param2);
	void ON_ERROR_CTRL(int nErrorCode);
	void ON_ERROR_VIDEO(int nErrorcode);
	void ON_ERROR_AUDIO_SPEAKER(int nErrorcode);
	void ON_ERROR_AUDIO_MIC(int nErrorcode);
};

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class CLiveObject
{
private:
	DWORD			m_dwMotionDetectInfo[MAX_MTD_ROW_COUNT];
	
	CLiveChannel	*m_ChannelInfo;

	//..................................................

public:
	CLiveObject();
	virtual ~CLiveObject();

	//..................................................

public:
	void OnConnect( char *szAddr, char *szControlPort, char *szVideoPort, char *szAudioTPort, char *szAudioRPort, char *szUserId, char *szPasswd );
	void OnDisConnect();
	void OnDestroy();

	//..................................................

	void SetClientScreen( int nChannel, HWND hWnd, int x, int y, DWORD dwWWidth, DWORD dwWHeight );
	void SetConnectStream( int nChannel, VIDEO_DATA_PACK *pVideoData, BOOL *pIsDecodingSuccess, WORD *pwPan, WORD *pwTilt, WORD *pwZoom, BYTE *byAlarmIn, BYTE *byAlarmOut );
	void SetVideoChannel( int nChannel );
	void SetMotionDetectionMode( int nChannel, int nMode );
	
	void AudioSpeakerOn( BOOL bFlag );
	
	void SetPTZSpeed( WORD wSpeed );
	void SetPTZMove( int nPan, int nTilt, double nZoom );
	void GetPTZCoordinates();
	int  SetEncoderSetupInfo( int nChannel, ENCODE_SETUP_INFO *lpData );
    void SetOutput( int nChannel, BOOL bOn, int nOutput );

	inline BOOL GetControlConnection()		{ return m_ChannelInfo->GetControlConnection(); };
	inline BOOL GetVideoConnection()		{ return m_ChannelInfo->GetVideoConnection(); };
	inline BOOL GetAudioSpeakerConnection()	{ return m_ChannelInfo->GetAudioSpeakerConnection(); };
	inline BOOL GetAudioMicConnection()		{ return m_ChannelInfo->GetAudioMicConnection(); };
	inline int  GetControlStatus()			{ return m_ChannelInfo->GetControlStatus(); };
	inline int  GetVideoStatus()			{ return m_ChannelInfo->GetVideoStatus(); };
	inline int  GetAudioSpeakerStatus()		{ return m_ChannelInfo->GetAudioSpeakerStatus(); };
	inline int  GetAudioMicStatus()			{ return m_ChannelInfo->GetAudioMicStatus(); };

	inline HANDLE GetHandle()				{ return m_ChannelInfo->GetHandle(); };
	inline BYTE  *GetOutImage()				{ return m_ChannelInfo->GetOutImage(); };
	inline int	  GetOutImageWidth()		{ return m_ChannelInfo->GetOutImageWidth(); };
	inline int    GetOutImageHeight()		{ return m_ChannelInfo->GetOutImageHeight(); };
	inline CAMERA_CONFIG GetCameraConfig()	{ return m_ChannelInfo->GetCameraConfig(); };

	inline BOOL GetAlarmInput1Status()		{ return m_ChannelInfo->GetAlarmInput1Status(); };
	inline BOOL GetAlarmINput2Status()		{ return m_ChannelInfo->GetAlarmINput2Status(); };
	inline BOOL GetMotionEvent()			{ return m_ChannelInfo->GetMotionEvent(); };
	inline int	GetAuthentication()			{ return m_ChannelInfo->GetAuthentication(); };
	
	BOOL GetEncoderSetupInfo( int nChannel, ENCODE_SETUP_INFO *lpData, BOOL bRefreshData );
	
	void GetAlarmInOutStatus();

	//..................................................

	void ButtonLeftUp( int nChannel );
	void ButtonUp( int nChannel );
	void ButtonRightup( int nChannel );
	void ButtonLeft( int nChannel );
	void ButtonRight( int nChannel );
	void ButtonLeftdown( int nChannel );
	void ButtonDown( int nChannel );
	void ButtonRightdown( int nChannel );
	void ButtonStop( int nChannel );
	void ButtonZoomIn( int nChannel );
	void ButtonZoomOut( int nChannel );
	
	//..................................................

	void ConnectVideo( BOOL bConnect );
	void ConnectSpeaker( BOOL bConnect );
	void ConnectMic( BOOL bConnect );

	//..................................................

	WAVEFORMATEX	GetAudioInFormat()		{ return m_ChannelInfo->GetAudioInFormat(); };
	WAVEFORMATEX	GetAudioOutFormat()		{ return m_ChannelInfo->GetAudioOutFormat(); };

	//..................................................

	void ShowMotionArea(BOOL bSHow);
	void SelectAllMotionArea();
	void ClearMotionArea();
	void SaveMotionArea();
	void SaveMotionSensitivity(int nValue);
	int  GetMotionSensitivity();
};

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#endif // !defined(AFX_LIVEDEMO_H__3FB62209_C384_4185_9A1C_B021214C85F1__INCLUDED_)
