// VideoDraw.cpp: implementation of the CVideoDraw class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "VideoDraw.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CVideoDraw::CVideoDraw()
{
	m_pWnd	= NULL;
	
	m_nWidth	= 0;
	m_nHeight	= 0;
	
	m_lpDD = NULL;				// The DirectDraw object
	ZeroMemory(&m_ddcaps, sizeof(m_ddcaps));
	m_lpClipper = NULL;
	
	m_surfacePrimary = NULL;	// DirectDraw primary surface
	m_surfaceRGB = NULL;		// RGB  back buffer surface

	m_isShowMotionArea = FALSE;
	memset(m_MotionSetupArea, 0, sizeof(m_MotionSetupArea));

	m_hLockMotionArea = CreateEvent(NULL, FALSE, TRUE, NULL);

	m_pMyBrush[0].CreateHatchBrush(HS_DIAGCROSS,RGB(0,0,255)  );
	m_pMyBrush[1].CreateHatchBrush(HS_DIAGCROSS,RGB(21,254,25)  );
	m_pMyBrush[2].CreateHatchBrush(HS_DIAGCROSS,RGB(98,157,17)  );
}

CVideoDraw::~CVideoDraw()
{
	Clear();

	if(m_hLockMotionArea)
	{
		CloseHandle(m_hLockMotionArea);
		m_hLockMotionArea = NULL;
	}
	if(m_pMyBrush[0].m_hObject)
	{
		m_pMyBrush[0].DeleteObject();
	}
	if(m_pMyBrush[1].m_hObject)
	{
		m_pMyBrush[1].DeleteObject();
	}
	if(m_pMyBrush[2].m_hObject)
	{
		m_pMyBrush[2].DeleteObject();
	}
}
void CVideoDraw::SurfaceColorFill(LPDIRECTDRAWSURFACE7  pSurf,RECT * pDst,COLORREF rgb)
{
	HRESULT ddrval;
	DDPIXELFORMAT ddpf;
	
	ddpf.dwSize = sizeof(ddpf);
	if(SUCCEEDED(pSurf->GetPixelFormat(&ddpf)))
	{
		DDBLTFX ddbltfx; 
		
		ddbltfx.dwSize = sizeof(ddbltfx); 
		//		ddbltfx.dwFillColor = ddpf.dwBBitMask; // Pure blue
		ddbltfx.dwFillColor = rgb; 
		ddrval = pSurf->Blt( 
			pDst,        // if NULL Destination is entire surface
			NULL,        // No source surface
			NULL,        // No source rectangle
			DDBLT_COLORFILL, &ddbltfx); 
		
		switch(ddrval) 
		{ 
		case DDERR_WASSTILLDRAWING: 
			TRACE("Color fill WASSTILLDRAWING\n");
			break;
		case DDERR_SURFACELOST: 
			TRACE("Color fill SURFACELOST\n");
			break;
		case DD_OK: 
			TRACE("Color fill OK\n");
			break;
		default: 
			break;
		} 
	}
}
void CVideoDraw::MakeRGBDesc(DDSURFACEDESC2 & desc)
{
    memset (&desc, 0, sizeof(DDSURFACEDESC2));
    desc.dwSize			= sizeof(DDSURFACEDESC2);
    desc.dwFlags		= DDSD_CAPS | DDSD_HEIGHT | DDSD_WIDTH| DDSD_PIXELFORMAT;
	desc.ddsCaps.dwCaps	= DDSCAPS_OFFSCREENPLAIN|DDSCAPS_VIDEOMEMORY;//|DDSCAPS_LOCALVIDMEM; // DDSCAPS_VIDEOMEMORY  DDSCAPS_OFFSCREENPLAIN
	desc.dwWidth		= 1920;
	desc.dwHeight		= 1080;
	
	desc.ddpfPixelFormat.dwSize				= sizeof(DDPIXELFORMAT);
	desc.ddpfPixelFormat.dwFlags			= DDPF_RGB;
	desc.ddpfPixelFormat.dwRGBAlphaBitMask	= 0x000000;
	
	//---------------------------------------------------------------------------------------
	DDPIXELFORMAT pixelFmt;
	memset (&pixelFmt, 0, sizeof(pixelFmt));
	pixelFmt.dwSize = sizeof(pixelFmt);
	
	
	HRESULT ddrval = m_surfacePrimary->GetPixelFormat (&pixelFmt);
	
	if (pixelFmt.dwFlags & DDPF_RGB)
	{
		// Handle 16 bits per pixel displays
		if (pixelFmt.dwRGBBitCount == 16)
		{
			if ((pixelFmt.dwRBitMask == 0xf800) &&(pixelFmt.dwGBitMask == 0x07e0) &&(pixelFmt.dwBBitMask == 0x001f))
			{
				//rgbPixelFormat = RGB565;
				desc.ddpfPixelFormat.dwRGBBitCount		= 16;
				desc.ddpfPixelFormat.dwRBitMask			= 0xf800;
				desc.ddpfPixelFormat.dwGBitMask			= 0x07e0;
				desc.ddpfPixelFormat.dwBBitMask			= 0x001f;
			}
			else if ((pixelFmt.dwRBitMask == 0x7c00) &&(pixelFmt.dwGBitMask == 0x03e0) &&(pixelFmt.dwBBitMask == 0x001f))
			{
				//rgbPixelFormat = RGB555;
				desc.ddpfPixelFormat.dwRGBBitCount		= 16;
				desc.ddpfPixelFormat.dwRBitMask			= 0x7c00;
				desc.ddpfPixelFormat.dwGBitMask			= 0x03e0;
				desc.ddpfPixelFormat.dwBBitMask			= 0x001f;
			}
		}
		
		// Handle 24 bits per pixel displays
		else if (pixelFmt.dwRGBBitCount == 24)
		{
			if ((pixelFmt.dwRBitMask == 0xff0000) &&(pixelFmt.dwGBitMask == 0x00ff00) &&(pixelFmt.dwBBitMask == 0x0000ff))
			{
			//	rgbPixelFormat = RGB24bpp;
				desc.ddpfPixelFormat.dwRGBBitCount		= 24;
				desc.ddpfPixelFormat.dwRBitMask			= 0xff0000;
				desc.ddpfPixelFormat.dwGBitMask			= 0x00ff00;
				desc.ddpfPixelFormat.dwBBitMask			= 0x0000ff;
			}
		}
		// Handle 32 bits per pixel displays
		else if (pixelFmt.dwRGBBitCount == 32)
		{
			if ((pixelFmt.dwRBitMask == 0xff0000) &&(pixelFmt.dwGBitMask == 0x00ff00) &&(pixelFmt.dwBBitMask == 0x0000ff))
			{
			//	rgbPixelFormat = RGB32bpp;
				desc.ddpfPixelFormat.dwRGBBitCount		= 32;
				desc.ddpfPixelFormat.dwRBitMask			= 0xff0000;
				desc.ddpfPixelFormat.dwGBitMask			= 0x00ff00;
				desc.ddpfPixelFormat.dwBBitMask			= 0x0000ff;
				
			}
		}
	}
	
}

BOOL CVideoDraw::InitRGBSurface()
{
	if (m_surfaceRGB != NULL)
	{
		m_surfaceRGB->Release();
		m_surfaceRGB=NULL;
	}
	
	DDSURFACEDESC2 surfaceDesc;
	HRESULT ddrval;
	static char errorMessage[200];
	
	MakeRGBDesc(surfaceDesc);
	//surfaceDesc.dwWidth		= m_nWidth;		//    ũ⸦ .
	//surfaceDesc.dwHeight		= m_nHeight;
	
    ddrval = m_lpDD->CreateSurface (&surfaceDesc, &m_surfaceRGB, NULL);
    if (ddrval != DD_OK)
    {
		return FALSE;
    }
	return TRUE;
}
void CVideoDraw::Init(HWND hWnd, DWORD dwWWidth, DWORD dwWHeight)
{
	m_pWnd=CWnd::FromHandle(hWnd);

	m_nWidth = dwWWidth;
	m_nHeight = dwWHeight;

    HRESULT ddrval;
	static char errorMessage[200];

	ddrval = DirectDrawCreateEx (NULL, (void**)&m_lpDD, IID_IDirectDraw7, NULL);
	if (ddrval != DD_OK)
	{
		Clear();
		return;
	}
	else
	{
		/*
		DDSURFACEDESC2 ddsc;
		memset(&ddsc,0,sizeof(ddsc));
		ddsc.dwSize=sizeof(ddsc);
		m_lpDD->GetDisplayMode(&ddsc);
		if((ddsc.ddpfPixelFormat.dwRGBBitCount != 16) && (ddsc.ddpfPixelFormat.dwRGBBitCount != 32))
		{
			m_lpDD->SetDisplayMode(ddsc.dwWidth,ddsc.dwHeight,16,0,0);
		}
		*/
	}

	ddrval = m_lpDD->SetCooperativeLevel (hWnd, DDSCL_NORMAL);
	if (ddrval != DD_OK)
	{
		Clear();
		return;			
	}

	DDSURFACEDESC2 surfaceDesc;
	memset (&surfaceDesc, 0, sizeof(surfaceDesc));
	surfaceDesc.dwSize			= sizeof(surfaceDesc);
	surfaceDesc.dwFlags			= DDSD_CAPS;
	surfaceDesc.ddsCaps.dwCaps	= DDSCAPS_PRIMARYSURFACE|DDSCAPS_VIDEOMEMORY|DDSCAPS_LOCALVIDMEM;
	
	ddrval = m_lpDD->CreateSurface (&surfaceDesc, &m_surfacePrimary, NULL);
	if (ddrval != DD_OK)
	{
		Clear();
		return;
	}

	DDPIXELFORMAT pixelFmt;
	memset (&pixelFmt, 0, sizeof(pixelFmt));
	pixelFmt.dwSize = sizeof(pixelFmt);
	ddrval = m_surfacePrimary->GetPixelFormat (&pixelFmt);
	if (ddrval != DD_OK)
	{
		Clear();
		return;
	}

	ddrval = m_lpDD->CreateClipper(0, &m_lpClipper, NULL);
	if (ddrval != DD_OK)
	{
		Clear();
		return;
	}

	ddrval = m_lpClipper->SetHWnd(0, hWnd);
	if (ddrval != DD_OK)
	{
		Clear();
		return;
	}

	ddrval = m_surfacePrimary->SetClipper (m_lpClipper);
	if (ddrval != DD_OK)
	{
		Clear();
		return;
	}
	
	if(!InitRGBSurface())
	{
		Clear();
		return;
	}
}
BOOL CVideoDraw::Start()
{
	return TRUE;
}
void CVideoDraw::Stop()
{

}
void CVideoDraw::Clear()
{
	if(m_surfaceRGB)
	{
		m_surfaceRGB->Release();
		m_surfaceRGB=NULL;
	}
	if(m_surfacePrimary)
	{
		m_surfacePrimary->Release();
		m_surfacePrimary=NULL;
	}
	if(m_lpClipper)
	{
		m_lpClipper->Release();
		m_lpClipper = NULL;
	}
	if (m_lpDD != NULL)
	{
		m_lpDD->Release();
		m_lpDD = NULL;
	}
}
void CVideoDraw::Render()
{
	CRect rectDest;
	m_pWnd->GetClientRect(&rectDest);
	m_pWnd->ClientToScreen(&rectDest);
	CRect rectSrc = CRect(0,0,m_nWidth,m_nHeight);

	HRESULT ddrval = m_surfacePrimary->Blt(&rectDest, m_surfaceRGB, &rectSrc,DDBLT_WAIT , NULL);
	if(ddrval != DD_OK)
	{
		TRACE("RENDER FAIL \n");
	}

}
void CVideoDraw::MoveWindow(DWORD dwWWidth, DWORD dwWHeight)
{
	//m_nWidth	= dwWWidth;
	//m_nHeight	= dwWHeight;
}

void CVideoDraw::DrawImage(BYTE *lpData, DWORD dwWidth, DWORD dwHeight, BYTE *lpMotionInfo, DWORD dwMotionInfoSize)
{
	if(m_nWidth != dwWidth || m_nHeight != dwHeight)
	{
		m_nWidth	= dwWidth;
		m_nHeight	= dwHeight;
	}

	HRESULT ddrval;
	DDSURFACEDESC2 surfaceDesc;
	memset(&surfaceDesc, 0, sizeof(surfaceDesc));
	surfaceDesc.dwSize = sizeof(surfaceDesc);

	if(m_surfaceRGB)
	{
		ddrval= m_surfaceRGB->Lock(NULL, &surfaceDesc, DDLOCK_WAIT, NULL);
		if(ddrval != DD_OK)
		{
			TRACE("LockSurface fail   : 0x%x!!\n",ddrval);
			return;
		}

		BYTE * pTmpDst =(BYTE *)(surfaceDesc.lpSurface);
		BYTE * pTmpSrc = lpData;
		int i, j, nPos;
		int posDest = 0, posSrc = 0;
		for(i = 0; i < m_nHeight; i++)
		{
			CopyMemory(pTmpDst + posDest,pTmpSrc + posSrc, dwWidth*4);
			posDest += surfaceDesc.lPitch;
			posSrc  += dwWidth*4;
		}
		ddrval= m_surfaceRGB->Unlock(NULL);
		if (ddrval != DD_OK)
		{
			return;
		}
		if(dwMotionInfoSize > 0)
		{
			DrawMotionArea(lpMotionInfo, dwMotionInfoSize);		
		}

		Render();

	}
}
void CVideoDraw::Refresh()
{
	SurfaceColorFill(m_surfaceRGB,NULL,OVERLAYMASK);
	
	CRect rectDest;
	m_pWnd->GetClientRect(&rectDest);
	m_pWnd->ClientToScreen(&rectDest);
	SurfaceColorFill(m_surfacePrimary,&rectDest,OVERLAYMASK);
}

void CVideoDraw::ShowMotionArea(BOOL bShow)
{
	m_isShowMotionArea = bShow;
}
void CVideoDraw::DrawMotionArea(BYTE *lpMotonInfo,  DWORD dwSize)
{
	if(!m_isShowMotionArea)
	{
		return;
	}
	HDC hDC;
	if(m_surfaceRGB->GetDC(&hDC) != DD_OK)
	{
		return;
	}
	int m_nRowCount, m_nColCount, m_nSrcHSIZE, m_nSrcVSIZE, m_nSrcHSKIP, m_nSrcVSKIP;
	int HSIZE, VSIZE;
	HSIZE = m_nWidth;
	VSIZE = m_nHeight;
	if((m_nWidth==176 && m_nHeight==128) ||
		(m_nWidth==176 && m_nHeight==120) ||
		(m_nWidth==176 && m_nHeight==144) )
	{
		m_nRowCount = 3;
		m_nColCount = 3;
		
		m_nSrcHSIZE = m_nWidth;	
		m_nSrcVSIZE = m_nHeight;
		
		m_nSrcHSKIP = HSIZE/m_nRowCount;
		m_nSrcVSKIP = VSIZE/m_nColCount;
	}	
	else if((m_nWidth==352 && m_nHeight==240) ||
		(m_nWidth==352 && m_nHeight==288))
	{
		m_nRowCount = 4;
		m_nColCount = 3;
		
		m_nSrcHSIZE = m_nWidth;	
		m_nSrcVSIZE = m_nHeight;
		
		m_nSrcHSKIP = HSIZE/m_nRowCount;
		m_nSrcVSKIP = VSIZE/m_nColCount;
	}	
	else if((m_nWidth==720 && m_nHeight==576) ||
		(m_nWidth==720 && m_nHeight==480) ||
		(m_nWidth==704 && m_nHeight==576) ||
		(m_nWidth==704 && m_nHeight==480) )
	{
		m_nRowCount = 9;
		m_nColCount = 6;
		
		m_nSrcHSIZE = m_nWidth;	
		m_nSrcVSIZE = m_nHeight;
		
		m_nSrcHSKIP = HSIZE/m_nRowCount;
		m_nSrcVSKIP = VSIZE/m_nColCount;
	}				
	else if(m_nWidth==1280 && m_nHeight==720)	// HD
	{
		m_nRowCount = 16;
		m_nColCount = 9;
		
		m_nSrcHSIZE = m_nWidth;	
		m_nSrcVSIZE = m_nHeight;
		
		m_nSrcHSKIP = HSIZE/m_nRowCount;
		m_nSrcVSKIP = VSIZE/m_nColCount;
	}
	else if(m_nWidth==1920 && (m_nHeight==1080 || m_nHeight==1072))	// full HD
	{
		m_nRowCount = 20;
		m_nColCount = 16;
		
		m_nSrcHSIZE = m_nWidth;	
		m_nSrcVSIZE = m_nHeight;
		
		m_nSrcHSKIP = HSIZE/m_nRowCount;
		m_nSrcVSKIP = VSIZE/m_nColCount;
	}
	int row, col;

	CDC dc;
	dc.Attach(hDC);
	dc.SetBkMode(TRANSPARENT);
	
	for( row=0; row<m_nRowCount; row++)
	{
		dc.MoveTo(m_nSrcHSKIP*row,     0);
		dc.LineTo(m_nSrcHSKIP*row, m_nSrcVSIZE);
	}
	for( col=0; col<m_nColCount; col++)
	{
		dc.MoveTo(0    , m_nSrcVSKIP*col);
		dc.LineTo(m_nSrcHSIZE, m_nSrcVSKIP*col);
	}

	BOOL bFlag = FALSE;
	CBrush * pOldBrush=NULL;
	for( col=0; col<m_nColCount; col++)
	{
		for( row=0; row<m_nRowCount; row++)
		{
			bFlag = FALSE;
			int i = row/8;
			int j = row%8;
			BYTE mark = lpMotonInfo[4*col + i];
			if(mark & (0x01<<(7-j)))
			{
				bFlag = TRUE;
			}
			
			if(bFlag)
			{
				// Motion deteting area mask
				if(m_MotionSetupArea[32*col+row])
				{
					pOldBrush=dc.SelectObject(&m_pMyBrush[0]);
					dc.Rectangle(CRect(m_nSrcHSKIP*row,m_nSrcVSKIP*col,
						m_nSrcHSKIP*row+m_nSrcHSKIP+1,m_nSrcVSKIP*col+m_nSrcVSKIP+1));
					dc.SelectObject(pOldBrush);
				}
			}
			else if(m_MotionSetupArea[32*col+row])	
			{
				// Motion setting area
				pOldBrush=dc.SelectObject(&m_pMyBrush[1]);
				dc.Rectangle(CRect(m_nSrcHSKIP*row,m_nSrcVSKIP*col,
					m_nSrcHSKIP*row+m_nSrcHSKIP+1,m_nSrcVSKIP*col+m_nSrcVSKIP+1));
				dc.SelectObject(pOldBrush);
			}
		}
	}
	dc.Detach();
	m_surfaceRGB->ReleaseDC(hDC);
}
void CVideoDraw::SetMotionInfo(BYTE *lpData,  DWORD dwSize)
{
	WaitForSingleObject(m_hLockMotionArea, INFINITE);
	if(dwSize <= 32*18)
	{
		memcpy(m_MotionSetupArea, lpData, dwSize);
	}
	SetEvent(m_hLockMotionArea);
}
BOOL CVideoDraw::GetMotionInfo(BYTE *lpData,  DWORD &dwSize)
{
	WaitForSingleObject(m_hLockMotionArea, INFINITE);
	dwSize = 72;
	memcpy(lpData, m_MotionSetupArea, dwSize);
	SetEvent(m_hLockMotionArea);
	return TRUE;
}